@@ -7,6 +7,13 @@ struct fs_file;
 struct fs_lock;
 struct hash_method;
 
+/* Metadata with this prefix shouldn't actually be sent to storage. */
+#define FS_METADATA_INTERNAL_PREFIX ":/X-Dovecot-fs-api-"
+/* fs_write*() may return a hex-encoded object ID after write is finished.
+   This can be later on used to optimize reads by setting it before reading
+   the file. */
+#define FS_METADATA_OBJECTID FS_METADATA_INTERNAL_PREFIX"ObjectID"
+
 enum fs_properties {
 	FS_PROPERTY_METADATA	= 0x01,
 	FS_PROPERTY_LOCKS	= 0x02,
@@ -25,7 +32,11 @@ enum fs_properties {
 	FS_PROPERTY_WRITE_HASH_SHA256	= 0x200,
 	/* fs_copy() will copy the metadata if fs_set_metadata() hasn't
 	   been explicitly called. */
-	FS_PROPERTY_COPY_METADATA	= 0x400
+	FS_PROPERTY_COPY_METADATA	= 0x400,
+	/* Backend support asynchronous file operations. */
+	FS_PROPERTY_ASYNC		= 0x800,
+	/* Backend supports FS_ITER_FLAG_OBJECTIDS. */
+	FS_PROPERTY_OBJECTIDS		= 0x1000
 };
 
 enum fs_open_mode {
@@ -69,7 +80,17 @@ enum fs_iter_flags {
 	/* Iterate only directories, not files */
 	FS_ITER_FLAG_DIRS	= 0x01,
 	/* Request asynchronous iteration. */
-	FS_ITER_FLAG_ASYNC	= 0x02
+	FS_ITER_FLAG_ASYNC	= 0x02,
+	/* Instead of returning object names, return <objectid>/<object name>.
+	   If this isn't supported, the <objectid> is returned empty. The
+	   object IDs are always hex-encoded data. This flag can be used only
+	   if FS_PROPERTY_OBJECTIDS is enabled. */
+	FS_ITER_FLAG_OBJECTIDS	= 0x04,
+	/* Explicitly disable all caching for this iteration (if anything
+	   happens to be enabled). This should be used only in situations where
+	   the iteration is used to fix something that is broken, e.g. doveadm
+	   force-resync. */
+	FS_ITER_FLAG_NOCACHE	= 0x08
 };
 
 struct fs_settings {
@@ -101,6 +122,39 @@ struct fs_settings {
 	bool debug;
 };
 
+struct fs_stats {
+	/* Number of fs_prefetch() calls. Counted only if fs_read*() hasn't
+	   already been called for the file (which would be pretty pointless
+	   to do). */
+	unsigned int prefetch_count;
+	/* Number of fs_read*() calls. Counted only if fs_prefetch() hasn't
+	   already been called for the file. */
+	unsigned int read_count;
+	/* Number of fs_lookup_metadata() calls. Counted only if neither
+	   fs_read*() nor fs_prefetch() has been called for the file. */
+	unsigned int lookup_metadata_count;
+	/* Number of fs_stat() calls. Counted only if none of the above
+	   has been called (because the stat result should be cached). */
+	unsigned int stat_count;
+
+	/* Number of fs_write*() calls. */
+	unsigned int write_count;
+	/* Number of fs_exists() calls, which actually went to the backend
+	   instead of being handled by fs_stat() call due to fs_exists() not
+	   being implemented. */
+	unsigned int exists_count;
+	/* Number of fs_delete() calls. */
+	unsigned int delete_count;
+	/* Number of fs_copy() calls. If backend doesn't implement copying
+	   operation but falls back to regular read+write instead, this count
+	   isn't increased but the read+write counters are. */
+	unsigned int copy_count;
+	/* Number of fs_rename() calls. */
+	unsigned int rename_count;
+	/* Number of fs_iter_init() calls. */
+	unsigned int iter_count;
+};
+
 struct fs_metadata {
 	const char *key;
 	const char *value;
@@ -112,8 +166,17 @@ typedef void fs_file_async_callback_t(void *context);
 int fs_init(const char *driver, const char *args,
 	    const struct fs_settings *set,
 	    struct fs **fs_r, const char **error_r);
+/* same as fs_unref() */
 void fs_deinit(struct fs **fs);
 
+void fs_ref(struct fs *fs);
+void fs_unref(struct fs **fs);
+
+/* Returns the parent filesystem (if this is a wrapper fs) or NULL if
+   there's no parent. */
+struct fs *fs_get_parent(struct fs *fs);
+/* Returns the filesystem's driver name. */
+const char *fs_get_driver(struct fs *fs);
 /* Returns the root fs's driver name (bypassing all wrapper fses) */
 const char *fs_get_root_driver(struct fs *fs);
 
@@ -132,6 +195,10 @@ void fs_set_metadata(struct fs_file *file, const char *key, const char *value);
 /* Return file's all metadata. */
 int fs_get_metadata(struct fs_file *file,
 		    const ARRAY_TYPE(fs_metadata) **metadata_r);
+/* Wrapper to fs_get_metadata() to lookup a specific key. Returns 1 if value_r
+   is set, 0 if key wasn't found, -1 if error. */
+int fs_lookup_metadata(struct fs_file *file, const char *key,
+		       const char **value_r);
 
 /* Returns the path given to fs_open(). If file was opened with
    FS_OPEN_MODE_CREATE_UNIQUE_128 and the write has already finished,
@@ -234,4 +301,9 @@ void fs_iter_set_async_callback(struct fs_iter *iter,
    function to determine if you should wait for more data or finish up. */
 bool fs_iter_have_more(struct fs_iter *iter);
 
+/* Return the filesystem's fs_stats. Note that each wrapper filesystem keeps
+   track of its own fs_stats calls. You can use fs_get_parent() to get to the
+   filesystem whose stats you want to see. */
+const struct fs_stats *fs_get_stats(struct fs *fs);
+
 #endif
