@@ -15,8 +15,10 @@ struct mail_storage_settings {
 	const char *mail_attachment_dir;
 	const char *mail_attachment_hash;
 	uoff_t mail_attachment_min_size;
+	const char *mail_attribute_dict;
 	unsigned int mail_prefetch_count;
 	const char *mail_cache_fields;
+	const char *mail_always_cache_fields;
 	const char *mail_never_cache_fields;
 	unsigned int mail_cache_min_mail_count;
 	unsigned int mailbox_idle_check_interval;
@@ -30,6 +32,7 @@ struct mail_storage_settings {
 	bool mail_nfs_storage;
 	bool mail_nfs_index;
 	bool mailbox_list_index;
+	bool mailbox_list_index_very_dirty_syncs;
 	bool mail_debug;
 	bool mail_full_filesystem_access;
 	bool maildir_stat_dirs;
@@ -37,6 +40,10 @@ struct mail_storage_settings {
 	const char *lock_method;
 	const char *pop3_uidl_format;
 
+	const char *ssl_client_ca_dir;
+	const char *ssl_client_ca_file;
+	const char *ssl_crypto_device;
+
 	enum file_lock_method parsed_lock_method;
 	enum fsync_mode parsed_fsync_mode;
 };
@@ -55,8 +62,9 @@ struct mail_namespace_settings {
 	bool subscriptions;
 	bool ignore_on_failure;
 	bool disabled;
+	unsigned int order;
 
-	ARRAY_DEFINE(mailboxes, struct mailbox_settings *);
+	ARRAY(struct mailbox_settings *) mailboxes;
 	struct mail_user_settings *user_set;
 };
 
@@ -69,6 +77,7 @@ struct mailbox_settings {
 	const char *name;
 	const char *autocreate;
 	const char *special_use;
+	const char *driver;
 };
 
 struct mail_user_settings {
@@ -92,8 +101,8 @@ struct mail_user_settings {
 
 	const char *mail_log_prefix;
 
-	ARRAY_DEFINE(namespaces, struct mail_namespace_settings *);
-	ARRAY_DEFINE(plugin_envs, const char *);
+	ARRAY(struct mail_namespace_settings *) namespaces;
+	ARRAY(const char *) plugin_envs;
 };
 
 extern const struct setting_parser_info mail_user_setting_parser_info;
@@ -110,9 +119,6 @@ const struct mail_storage_settings *
 mail_user_set_get_storage_set(struct mail_user *user);
 const void *mail_storage_get_driver_settings(struct mail_storage *storage);
 
-enum mail_index_open_flags
-mail_storage_settings_to_index_flags(const struct mail_storage_settings *set);
-
 const struct dynamic_settings_parser *
 mail_storage_get_dynamic_parsers(pool_t pool);
 
